/* +-----------------------------------------------------------------+
   | FSerrmsg.c    define and print error messages in FlagShip       |
   |                                                                 |
   | Copyright 1996 by multisoft GmbH, Munich                        |
   +-----------------------------------------------------------------+ */

#if defined(FGSAIX)
#   include <cur01.h>
#else
#   include <curses.h>
#endif

#include "FSerrors.h"

extern int errmsg_row,              /* row for error messages          */
           errmsg_col;              /* column for error messages       */

char _blank_err_msg[999] = { ' ' }; /* a blank line for error messages */

WINDOW *ERR_WIND = 0,               /* ERROR messages display window   */
       *ERR_FRAME = 0;              /* ERROR window frame holder       */

struct __error_report __error_report[MAXERRMSG];

#define _ERROR_SET(x, y, z)   { __error_report[x].err_msg = y;\
                                __error_report[x].when = z; }


/* +-----------------------------------------------------------------+
   | err_init:                                                       |
   |                                                                 |
   | This routine is called automatically called at initialization   |
   | time to initialize the error message structures and to prepare  |
   | the error mesage windows.                                       |
   |                                                                 |
   +-----------------------------------------------------------------+ */

void
err_init()
{
   int i;

   /* define ERROR msg window, frame                                   */

   if(stdscr)
   {
	   ERR_FRAME = newwin(12, 54, errmsg_row, errmsg_col);
	   ERR_WIND  = subwin(ERR_FRAME, 8, 50, errmsg_row+2, errmsg_col+2);
	   werase(ERR_FRAME);
	   mvwaddstr(ERR_FRAME, 0, 0,
				 "******************************************************");
	   for(i=1; i<12;  i++)
	   {
		   mvwaddch(ERR_FRAME, i, 0, '*');
		   mvwaddch(ERR_FRAME, i, 53, '*');
	   }
	   mvwaddstr(ERR_FRAME, 11, 0,
				 "************* Arrows move, <ESC> exits ***************");
   }
   

   /* length of a blank error line                                    */
   for (i=0; i<COLS; i++) _blank_err_msg[i] = ' ';
   _blank_err_msg[COLS] = 0;

   /* initialize default error messages                               */
   for (i=0; i<MAXERRMSG; i++) _ERROR_SET(i, "???", RP_INDEVELOP);

/* +-----------------------------------------------------------------+
   | you may change the text and/or developer/user status of each    |
   | message, but be careful not to change the functionality istself.|
   |                                                                 |
   | syntax: _ERROR_SET (err_no, err_text, err_status);              |
   |                                                                 |
   |         err_no     (int)    = see FSerrors.h                    |
   |         err_text   (char *) = text for error screen             |
   |         err_status (int)    = see FSerrors.h                    |
   +-----------------------------------------------------------------+ */

   _ERROR_SET(IOOPEN,      "file open error",                 RP_ALWAYS);
   _ERROR_SET(IOCLOS,      "file close error",                RP_ALWAYS);
   _ERROR_SET(IOREAD,      "file read error",                 RP_ALWAYS);
   _ERROR_SET(IOWRIT,      "file write error",                RP_ALWAYS);
   _ERROR_SET(IOCREAT,     "create file error",               RP_ALWAYS);
   _ERROR_SET(IOERASE,     "erase file error",                RP_ALWAYS);
   _ERROR_SET(IOSTAT,      "file status error",               RP_ALWAYS);

   _ERROR_SET(RECNOOR,     "record number out of range",      RP_ALWAYS);

   _ERROR_SET(DTMMACH,     "data type mismatch",              RP_ALWAYS);
   _ERROR_SET(DTILOPR,     "illegal operation on data type",  RP_ALWAYS);
   _ERROR_SET(DTILEGL,     "illegal data type",               RP_ALWAYS);
   _ERROR_SET(DTILLFL,     "illegal field type",              RP_ALWAYS);
   _ERROR_SET(DTILVAL,     "illegal field value",             RP_ALWAYS);

   _ERROR_SET(RTNOMEM,     "no memory available",             RP_ALWAYS);
   _ERROR_SET(RTNOVAR,     "no memory variables available",   RP_ALWAYS);
   _ERROR_SET(RTNLMVP,     "null pointer for memory variable",RP_ALWAYS);
   _ERROR_SET(RTXPMPT,     "too many prompts for menu",       RP_ALWAYS);
   _ERROR_SET(RTMXGET,     "too many gets",                   RP_ALWAYS);
   _ERROR_SET(RTNOFLD,     "cannot find that field",          RP_ALWAYS);
   _ERROR_SET(RTCONVAR,    "constant variable",               RP_ALWAYS);
   _ERROR_SET(RTUNVAR,     "undefined variable",              RP_ALWAYS);
   _ERROR_SET(RTUNFUN,     "undefined function",              RP_ALWAYS);
   _ERROR_SET(RTSYNTX,     "Syntax error in expression",      RP_ALWAYS);
   _ERROR_SET(RTEEVAL,     "NULL pointer returned from evaluator", RP_ALWAYS);
   _ERROR_SET(RTARGCNT,    "Wrong argument count",            RP_ALWAYS);
   _ERROR_SET(RTARGTYP,    "Wrong argument type",             RP_ALWAYS);
   _ERROR_SET(RTARGVAL,    "Wrong argument value",            RP_ALWAYS);
   _ERROR_SET(RTENDLMEXP,  "Endless loop in macro expansion", RP_ALWAYS);
   _ERROR_SET(RTTLMEXP,    "Too long macro expansion",        RP_ALWAYS);
   _ERROR_SET(RTMISARG,    "Missing argument",                RP_ALWAYS);
   _ERROR_SET(RTILLTYP,    "Illegal type",                    RP_ALWAYS);
   _ERROR_SET(RTILLCOL,    "Illegal color specification",     RP_INDEVELOP);
   _ERROR_SET(RTILLVAL,    "Illegal value",                   RP_ALWAYS);
   _ERROR_SET(RTIDXCRPT,   "Index file corrupted",            RP_ALWAYS);
   _ERROR_SET(RTIVSKEL,    "Illegal skeleton",                RP_ALWAYS);
   _ERROR_SET(RTNODBF,     "No database opened",              RP_ALWAYS);
   _ERROR_SET(RTNODBF1,    "No database opened",              RP_INDEVELOP);
   _ERROR_SET(RTFILNAM,    "Wrong file name",                 RP_ALWAYS);
   _ERROR_SET(RTARGNLS,    "Argument is null string",         RP_ALWAYS);
   _ERROR_SET(RTNOLOCK,    "DBF file or record not locked",   RP_ALWAYS);
   _ERROR_SET(RTNOMORLCKS, "Number of locks for the DBF exceeded", RP_ALWAYS);
   _ERROR_SET(RTWRNGLIN,   "Incorrect line in file",          RP_ALWAYS);
   _ERROR_SET(RTILLPICT,   "Illegal picture string",          RP_INDEVELOP);
   _ERROR_SET(RTILLDATE,   "Illegal dateformat string",       RP_INDEVELOP);
   _ERROR_SET(RTILLEOF,    "Record wanted but eof",           RP_ALWAYS);
   _ERROR_SET(RTDBFVER,    "Wrong version of DBF file",       RP_ALWAYS);
   _ERROR_SET(RTILLBRK,    "BREAK unexpected",                RP_ALWAYS);
   _ERROR_SET(RTOSERR,     "command or filename not found",   RP_NEVER);
   _ERROR_SET(RTSTRTL,     "string too long",                 RP_ALWAYS);
   _ERROR_SET(RTILLNAM,    "Undefined identifer",             RP_ALWAYS);
   _ERROR_SET(RTILLCLS,    "Illegal object class",            RP_ALWAYS);
   _ERROR_SET(RTUSERS,     "unable to open file",             RP_ALWAYS);
   _ERROR_SET(RTIDXVER,    "Wrong IDX version of file",       RP_ALWAYS);
   _ERROR_SET(RTNOFLD1,    "cannot find that field",          RP_INDEVELOP);
   _ERROR_SET(RTILLALIAS,  "Illegal alias",                   RP_ALWAYS);
   _ERROR_SET(RTILLMET,    "Illegal method",                  RP_ALWAYS);
   _ERROR_SET(RTILLINS,    "Illegal instance",                RP_ALWAYS);
   _ERROR_SET(RTLOCKFAIL,  "fcntl() lock failure",            RP_ALWAYS);

   _ERROR_SET(INTARGCN,    "argcnt negative",                 RP_ALWAYS);
   _ERROR_SET(INTARGCB,    "too many args",                   RP_ALWAYS);
   _ERROR_SET(INTARGCL,    "too little args",                 RP_ALWAYS);
   _ERROR_SET(INTARGTYP,   "wrong arg type",                  RP_ALWAYS);
   _ERROR_SET(INTARGNEG,   "arg negative",                    RP_ALWAYS);
   _ERROR_SET(INTARGNUL,   "arg null",                        RP_ALWAYS);
   _ERROR_SET(INTARGNLP,   "arg null pointer",                RP_ALWAYS);
   _ERROR_SET(INTARGNLS,   "arg points to null string",       RP_ALWAYS);
   _ERROR_SET(INTARGOR,    "arg out of range",                RP_ALWAYS);
   _ERROR_SET(INTCMDFLG,   "flag has impossible value",       RP_ALWAYS);
   _ERROR_SET(INTMEMOUT,   "out of memory",                   RP_ALWAYS);
   _ERROR_SET(INTNDXSEC,   "non-existing index data sector access",RP_ALWAYS);
   _ERROR_SET(INTDBFNO,    "database file (DBF) not opened ",      RP_ALWAYS);
   _ERROR_SET(INTDBTNE,    "database file has no memo file (DBT) ",RP_ALWAYS);
   _ERROR_SET(INTNDXNO,    "database index file (IDX) not opened ",RP_ALWAYS);
   _ERROR_SET(INTILLVAL,   "illegal internal value",          RP_ALWAYS);
   _ERROR_SET(LOCKNDX,     "index file already locked",       RP_ALWAYS);

   _ERROR_SET(FTNOTOPN,    "unable to open file",             RP_ALWAYS);
   _ERROR_SET(FTFUNARG,    "too many function arguments",     RP_ALWAYS);

   _ERROR_SET(GENRAL,      "",                                RP_ALWAYS);
   _ERROR_SET(GENWARNING,  "(warning)",                       RP_ALWAYS);
   _ERROR_SET(DEVWARNING,  "(warning)",                       RP_INDEVELOP);
}

/*** EOF FSerrmsg.c ***/

