/* +-----------------------------------------------------------------+
   | FSerror.c   FlagShip error system in/output routines            |
   |             see also FSerrmsg.c and FSerrors.h                  |
   |                                                                 |
   | Copyright 1996 by multisoft GmbH, Munich                        |
   +-----------------------------------------------------------------+ */

#include <stdio.h>
#include <errno.h>

#include "FlagShip.h"
#include "FSerrors.h"
#if defined(FGSAIX)
#   include <cur02.h>
#endif

#define PROG_PHASE 0
extern int fs_set_flag[];

/* system undeclared functions                                          */
extern void fgs_doerr_out_ref();
extern void fgs_doerr_end_ref();
#if ! defined(FGSHPUX) || ! defined(FGSAIX)
extern int getbegy();
extern int getbegx();
extern int getmaxy();
extern int getmaxx();
extern int getcury();
extern int getcurx();
#endif


extern void       push_scrpos ();
extern void       pop_scrpos ();
extern int        _pop_tty ();
extern int        _pop_vidattr ();
extern int        _push_tty ();
extern int        _push_vidattr ();
/* system undeclared functions                                          */

extern int errmsg_row,        /* row for error messages                      */
           errmsg_col;        /* column for error messages                   */

extern int __diag;            /* wether to print diagnostic messages         */
extern int __diag_level;      /* diagnostic messages level                   */
extern char *__diag_types;    /* diagnostic messages types                   */
extern char  __diag_buf[];    /* diag message output buffer                  */

extern int last_errno;        /* last error number                           */
extern int last_errtyp;       /* last error type                             */
extern int errstat;           /* error status (1= display errors, 0= don't)  */

extern struct                 /* array of pointers to error messages         */
   __error_report __error_report[];



extern WINDOW *ERR_WIND,      /* ERROR messages display window               */
              *ERR_FRAME;     /* ERROR window frame                          */

void show_error();

/* check error priority and act accordingly                                  */

#define CHECK_DO_IMPORTANCE(x) \
   if(__error_report[x].when < fs_set_flag[PROG_PHASE]) return(FALSE_VAR)

#define ERRMSG(x) __error_report[x].err_msg


/* ---------------- error handling modules -----------------------------    */

/* I/O error handling routine                                               */

variable *io_error (errno, data)
int errno;                 /* Error code number                             */
char *data;                /* Error context description data                */
{
   char dum[999];         /* temporary space for err msg preparation        */

   CHECK_DO_IMPORTANCE(errno);

   werase (ERR_WIND);


   sprintf(dum, "I/O error #%d", errno);
   mvwaddstr (ERR_WIND, 0, 0, dum);

   sprintf(dum, "File: %s, func/proc %s, line %d",
      old_fn_stack->prg_name, old_fn_stack->module_name, old_fn_stack->lin_no);
   mvwaddstr (ERR_WIND, 1, 0, dum);

   sprintf(dum, "Error: %s, %s", ERRMSG(errno), data);
   mvwaddstr (ERR_WIND, 2, 0, dum);

   last_errno = errno;
   last_errtyp = ERRTYP_IO;

   show_error();
   return(ERROR_VAR);
}


/* Run-time error handling routine                                          */

variable *rt_error (errno, data)
int errno;                 /* Error code number                             */
char *data;                /* Error context description data                */
{
   char dum[999];         /* temporary space for err msg preparation        */

   CHECK_DO_IMPORTANCE(errno);

   werase (ERR_WIND);

   sprintf(dum, "Run-time error #%d", errno);
   mvwaddstr (ERR_WIND, 0, 0, dum);

   sprintf(dum, "File: %s, func/proc %s, line %d",
      old_fn_stack->prg_name, old_fn_stack->module_name, old_fn_stack->lin_no);
   mvwaddstr (ERR_WIND, 1, 0, dum);

   sprintf(dum, "Error: %s, %s", ERRMSG(errno), data);
   mvwaddstr (ERR_WIND, 2, 0, dum);

   last_errno = errno;
   last_errtyp = ERRTYP_RT;

   show_error();
   return(ERROR_VAR);
}


/* Fatal error handling routine                                             */

variable *ftl_error (errno, data)
int errno;                 /* Error code number                             */
char *data;                /* Error context description data                */
{
   char dum[999];         /* temporary space for err msg preparation        */

   CHECK_DO_IMPORTANCE(errno);

   werase (ERR_WIND);


   sprintf(dum, "FATAL ERROR #%d", errno);
   mvwaddstr (ERR_WIND, 0, 0, dum);

   sprintf(dum, "File: %s, func/proc %s, line %d",
      old_fn_stack->prg_name, old_fn_stack->module_name, old_fn_stack->lin_no);
   mvwaddstr (ERR_WIND, 1, 0, dum);

   sprintf(dum, "Error: %s, %s", ERRMSG(errno), data);
   mvwaddstr (ERR_WIND, 2, 0, dum);

   mvwaddstr (ERR_WIND, 6, 0, " Program will be terminated !!! ");

   last_errno = errno;
   last_errtyp = ERRTYP_FTL;

   show_error();

   fgs_quit(); /* TERMINATE THE PROGRAM                               */
}


/* Internal error handling routine                                          */

variable *int_error (errno, data, file_name, file_line)
int errno;           /* error (message) number, used to print message       */
char *data;          /* context specific message                            */
char *file_name;     /* file name of module reporting error                 */
int file_line;       /* line number of file with module reporting error     */
{
   char dum[999];         /* temporary space for err msg preparation        */

   CHECK_DO_IMPORTANCE(errno);

   werase (ERR_WIND);

   sprintf(dum, "INTERNAL ERROR #%d, file %s, line %s",
      errno, file_name, file_line);
   mvwaddstr (ERR_WIND, 0, 0, dum);

   sprintf(dum, "File: %s, func/proc %s, line %d",
      old_fn_stack->prg_name, old_fn_stack->module_name, old_fn_stack->lin_no);
   mvwaddstr (ERR_WIND, 1, 0, dum);

   sprintf(dum, "Error: %s, %s", ERRMSG(errno), data);
   mvwaddstr (ERR_WIND, 2, 0, dum);

   last_errno = errno;
   last_errtyp = ERRTYP_INT;

   show_error();
   return(ERROR_VAR);
}


/* Error window management routine
   ... will be called from various error handling routines
   ... presents the error window
   ... allows user to move the window pressing arrow keys
   ... waits for ESC to be depressed to remove the error windows            */

void show_error ()
{
    int c = 0;
    static int alreadyin = 0;

    extern int _filbuf();

   if(alreadyin > 1) exit(-1);
   if(alreadyin++)
   {
      endwin();
      printf("\rFATAL ERROR in file %s, module %s, line %d\nPress any key...\n",
        old_fn_stack->prg_name, old_fn_stack->module_name,old_fn_stack->lin_no);

      getchar();
      fgs_quit();
   }


   if (errstat == 0)
   {
      --alreadyin;
      return;
   }

   /* save the screen and teminal status                                    */
   /* needs to be done because new windows with standard settings are       */
   /* being displayed                                                       */

   _push_tty ();                /* FS internal terminal settings stack      */
#  if defined(FGSAIX)
   crmode();
#  else
   cbreak();
#  endif
   noecho();
   fgs_delay(stdscr);
   push_scrpos();               /* FS internal screen position stack        */
   _push_vidattr();             /* FS internal screen video attribute stack */
   _bb__standard (0, 0);        /* standard window color settings           */

   /* prepare the frame around the message                                  */
   box (ERR_FRAME, '*', '*');
   mvwaddstr (ERR_FRAME, 11, 13, "  Press <ESC> to continue  ");

   /* display the error message                                             */
   fgs_doerr_out_ref(stdscr, ERR_FRAME, ERR_WIND, 2,1,1);

   /* handle the user response alowing the movement of the error window     */
   while (c != ESC) {
      c = getch();
      switch (c) {
         case ESC:
            break;
         case KEY_UP:
            if (errmsg_row > 0) { errmsg_row--; }
            break;
         case KEY_DOWN:
            if (errmsg_row < LINES-12-1) { errmsg_row++; }
            break;
         case KEY_LEFT:
            if (errmsg_col > 0) { errmsg_col--; }
            break;
         case KEY_RIGHT:
            if (errmsg_col < COLS-54-1) { errmsg_col++; }
            break;
         default:
            break;
      }

      /* move and redisplay the error window                                */
      mvwin (ERR_FRAME, errmsg_row, errmsg_col);
      mvwin (ERR_WIND, errmsg_row+2, errmsg_col+2);
      fgs_doerr_out_ref(stdscr, ERR_FRAME, ERR_WIND, 1, 0, 0);
   }

   /* restore the screen and terminal state back to normal                  */
   _pop_vidattr();      /*  FS internal video attribute stack               */

   pop_scrpos();        /* FS internal screen position stack                */
   _pop_tty ();         /* FS internal tty setting                          */

   fgs_doerr_end_ref(stdscr, 1);
   --alreadyin;
}

void fs_break_error()
{
    rt_error(RTILLBRK, "Break without a BEGIN SEQUENCE");
    fgs_quit();
}

/* :FILEND= fserror.c ===================================================   */

