/****************************************************************************
 * FSerror.prg    Error handler interface for FlagShip.                     *
 *                                                                          *
 * Warning: modifying the C code requires a good knowledge of C programming *
 *          and the Open C System of FlagShip. Otherwise, do not modify     *
 *          the program part within #Cinline and #endCinline.               *
 *                                                                          *
 * Compile: FlagShip -na -nd -nl -nL -c FSerror.prg                         *
 *                                      (in LOCAL working directory)        *
 * Use    : FlagShip myapplic*.prg FSerror.o                                *
 *--------------------------------------------------------------------------*
 * (c) 1998 multisoft Datentechnik GmbH, Germany.                           *
 ****************************************************************************/

/* #define INCLUDE_PRINT                 disabled by default      */

#include "error.fh"
#include "errclass.fh"

#stdout Note:Warnings due to missing prototypes may be ignored

#Cinline
#include <fcntl.h>
#include "FSopenc.h"
#include "FSerrors.h"

extern void exit();
extern void fgs_quit();
extern int __sig_quit;
extern struct                       /* array of pointers to error messages */
   __error_report __error_report[];

extern int last_errno;      /* last error number                           */
extern int errstat;         /* error status (1= display errors, 0= don't)  */

/* extern int _filbuf(); extern int endwin()   */
/* not required, not available on every system */

static int alreadyin = 0;   /* recursion check flag                        */

/*----------------------------------------------------------------*
 | recursive error check                                          |
 | this check avoids recursive calls to the errorsystem,          |
 |  (prevent segmentation when error during errorblock-evaluation)|
 *----------------------------------------------------------------*/
static void err_check_recursion(flag)
int flag;                          /* do not change this function, else    */
{                                  /* an error occuring in the err handler */
                                   /* can only result in a segmentation    */
   int ii;                         /* violation due to stack overflow      */
   fn_stack * stack;               /* can only result in a segmentation    */

   if (!flag) {
     alreadyin --;
     return;
   }
   if(alreadyin > 4) {             /* Error during quit, no way to get out */
	 if(stdscr) {  
	    nodelay (stdscr, FALSE);
        endwin();
     }
     exit(-1);
   }
   if(alreadyin++ > 3) {           /* Error during error, recursion        */

      printf("\rFATAL ERROR in module %s %d: %s()\r\n",
      old_fn_stack->prg_name, old_fn_stack->lin_no, old_fn_stack->module_name);

      printf("Called from:\r\n");
      for (stack = old_fn_stack->old_fn_stack; stack && stack->old_fn_stack;
                                               stack = stack->old_fn_stack) {
          if (stack && stack->lin_no >= 0)
              printf("%10s %3d: %10s()\r\n",
                     stack->prg_name, stack->lin_no, stack->module_name);
      }
      printf("Press return ...\r\n");

     stdscr ? getch() : getchar();
     fgs_quit();
   }
}

/*------------------------*
 | Run time error         |
 *------------------------*/
FSvar *rt_error (errno, data)
int errno;                         /* Error code number                    */
char *data;                        /* Error context description data       */
{
   FSvar * argv[3];
   err_check_recursion(1);
   last_errno  = errno;
   if (errstat != 0) {
      argv[0] = SET_VAR_NUM(VAR_NEW, (double) errno);
      argv[1] = SET_VAR_NUM(VAR_NEW, (double) ERRTYP_RT);
      argv[2] = SET_VAR_CHR(VAR_NEW, data);
      SET_VAR_NUM(VAR_NEW, (double) errno);
      fsFnNAME(_errcaller)(3, argv);
   }
   err_check_recursion(0);
   return(ERROR_VAR);
}
/*------------------------*
 | Illegal break error    |
 *------------------------*/
void fs_break_error()
{
   rt_error(RTILLBRK, "Break without a BEGIN SEQUENCE");
   fgs_quit();
}

/*------------------------*
 | Fatal error            |
 *------------------------*/
FSvar *ftl_error (errno, data)
int errno;                         /* Error code number                    */
char *data;                        /* Error context description data       */
{
   FSvar * argv[3];
   err_check_recursion(1);
   last_errno  = errno;
   if (errstat != 0) {
      argv[0] = SET_VAR_NUM(VAR_NEW, (double) errno);
      argv[1] = SET_VAR_NUM(VAR_NEW, (double) ERRTYP_FTL);
      argv[2] = SET_VAR_CHR(VAR_NEW, data);
      SET_VAR_NUM(VAR_NEW, (double) errno);
      fsFnNAME(_errcaller)(3, argv);
   }
   err_check_recursion(0);
   fgs_quit();
   return (FSvar *) 0;
}

/*------------------------*
 | IO error               |
 *------------------------*/
FSvar *io_error (errno, data)
int errno;                           /* Error code number                  */
char *data;                          /* Error context description data     */
{
   FSvar * argv[3];
   err_check_recursion(1);
   last_errno  = errno;
   if (errstat != 0) {
      argv[0] = SET_VAR_NUM(VAR_NEW, (double) errno);
      argv[1] = SET_VAR_NUM(VAR_NEW, (double) ERRTYP_IO);
      argv[2] = SET_VAR_CHR(VAR_NEW, data);
      SET_VAR_NUM(VAR_NEW, (double) errno);
      fsFnNAME(_errcaller)(3, argv);
   }
   err_check_recursion(0);
   return(ERROR_VAR);
}


/*------------------------*
 | Internal error         |
 *------------------------*/
FSvar *int_error (errno, data, file_name, file_line)
int errno;              /* error (message) number, used to print message   */
char *data;             /* context specific message                        */
char *file_name;        /* file name of module reporting error             */
int file_line;          /* line number of file with module reporting error */
{
   FSvar * argv[5];
   err_check_recursion(1);
   last_errno  = errno;
   if (errstat != 0) {
      argv[0] = SET_VAR_NUM(VAR_NEW, (double) errno);
      argv[1] = SET_VAR_NUM(VAR_NEW, (double) ERRTYP_INT);
      argv[2] = SET_VAR_CHR(VAR_NEW, data);
      argv[3] = SET_VAR_NUM(VAR_NEW, (double) file_line);
      argv[4] = SET_VAR_CHR(VAR_NEW, file_name);
      fsFnNAME(_errcaller)(5, argv);
   }
   err_check_recursion(0);
   return(ERROR_VAR);
}

#endCinline


/*----------------------------------------------------------------*
 | call error handler                                             |
 *----------------------------------------------------------------*/
STATIC FUNCTION _errcaller(e_code, e_sever, e_decr, e_line, e_file)
local erro := errornew(), msg, descript, aDescr, aTmp
local_int ret := 0,  ii, jj, kk

// The very first thing wwe do here is to check if _errcaller is already
// on the stack in case the above static C variable alreadyin is > 1.
// If not, we assume the previous errorblock call to be broken by the
// FlagShip BREAK() function.


#Cinline
   ret = alreadyin;
#endCinline

IF ret > 1                         // "recursive error" condition given
   ii := 2
   DO WHILE .T.
     msg := TRIM(PROCNAME(ii++))
     IF EMPTY(msg)                 // false alarm, BREAK happened
#        Cinline
         alreadyin --;
#        endCinline
       EXIT
     ENDIF
      IF (msg) == "_ERRCALLER"     //  truly recursive
       EXIT
        ENDIF
   ENDDO
ENDIF

#Cinline
   /* Get message of errorcode into msg and info if error disabled into ret */
   SET_VAR_CHR(VAR_NAME_LOCAL(msg),
   __error_report[ret = ((int) VAR_NUM(VAR_NAME_LOCPAR(e_code)))].err_msg);
   if(__error_report[ret].when < fs_set_flag[0]) ret = 1;
   else ret = 0;
#endCinline

if ret == 1            // Error disabled (depends on current FS_set("devel"))
   return NIL
endif
e_decr := strtran(e_decr, chr(10), ";")
aDescr := __splittxt(e_decr)
descript = ""
for ii := 1 to len(aDescr)
   if len(aDescr[ii]) > 76
     jj := at (",", aDescr[ii])
     if jj > 0 .and. jj < len(aDescr)
         aDescr[ii] = strtran(aDescr[ii], ",", ",;")
         aTmp := __splittxt(aDescr[ii])
       ASIZE (aDescr, len(aDescr) + len(aTmp) -1)
       aDescr[ii] := aTmp[1]
       for jj := 2 to len(aTmp)
         kk := ii + jj - 1
         AINS (aDescr, kk)
         aDescr[kk] := aTmp[jj]
       next
      endif
      if len(aDescr[ii]) > 76
       aDescr[ii] := left(aDescr[ii],56) + " ... " + right(aDescr[ii],15)
     endif
   endif
   descript += aDescr[ii] + if (ii < len(aDescr), ";", "")
next

erro:ARGS          := {}
erro:CANDEFAULT    := .F.
erro:CANRETRY      := .F.
erro:CANSUBSTITUTE := .F.
erro:CARGO         := NIL
erro:DESCRIPTION   := msg + ";" + descript
erro:FILENAME      := ""
erro:GENCODE       := e_code
erro:OPERATION     := ""
erro:OSCODE        := DOSERROR()
erro:SEVERITY      := e_sever
erro:SUBCODE       := IF(e_line == NIL,  0, e_line)
erro:SUBSYSTEM     := IF(e_file == NIL, "BASE", e_file)
erro:TRIES         := 0

msg := ERRORBLOCK()
IF (VALTYPE(msg) != "B")
   msg := { |x| _DEFERROR(x)}
ENDIF
EVAL(msg, erro)                         // Pass error to curr. error handler
return (NIL)


/*----------------------------------------------------------------*
 | default error handler, user modifiable                         |
 *----------------------------------------------------------------*/
FUNCTION _DEFERROR(e_obj)

LOCAL option := {}
LOCAL optlist:= {"Abort", "Ignore", "Callstack", "Debug", ;
                 "Print", "Errormessage"}
LOCAL choice := 0, replopt := 0, prc, ii, cc, old, msg_str2, msg_str
LOCAL sav_print, sav_cons, is_develop := .F.
LOCAL old_row := row(), old_col := col(), color := NIL
LOCAL iCursesOn AS INT
#define START_STACK  2

IF VALTYPE(e_obj) != "O"
    msg_str := "UNKNOWN ERROR"
ELSE
   IF(e_obj:SEVERITY != ES_FTLERR)
      IF e_obj:SEVERITY == ES_RTERR
         msg_str := "Run-time"
      ELSEIF e_obj:SEVERITY == ES_IOERR
         msg_str := "I/O"
      ELSE
         msg_str := "Internal"
      ENDIF
   ELSE
      msg_str := "Fatal"
   ENDIF
   msg_str += " error "
   if e_obj:GENCODE == EG_DEVWARNING               // Developer Warning ?
      if e_obj:SEVERITY == ES_RTERR .or.;
         e_obj:SEVERITY == ES_IOERR
         if !fs_set("develop")                     // fs_set("devel") set ?
            return NIL                             // no, ignore output
         endif
      endif
      msg_str += "(Developer warning) "
      is_develop := .T.
   else
      msg_str += str(e_obj:GENCODE,3)
   endif
   if !empty(e_obj:FILENAME)
      msg_str += " in " + e_obj:FILENAME
   else
      msg_str += " in " + PROCNAME(2) + "(" + LTRIM(STR(PROCLINE(2),5)) + ")"
   endif
   msg_str +=  ";" + e_obj:DESCRIPTION

   IF e_obj:SUBCODE != 0                     // info about subsystem available
      msg_str += ";" + e_obj:SUBSYSTEM
      IF e_obj:SUBCODE != -9999              // display subsystem number?
        msg_str += " (" +  LTRIM(STR(e_obj:SUBCODE,3)) + ")"
      ENDIF
   ENDIF
ENDIF

#Cinline
	icurseson = (FSint) stdscr;
#endCinline

IF iCursesOn == 0
	? msg_str	
	QUIT
ENDIF

#ifdef INCLUDE_PRINT
   AADD (option, optlist[5])                                // Print
#endif

if is_develop
   AADD (option, optlist[2])                                // Ignore
   AADD (option, optlist[1])                                // Abort
   AADD (option, optlist[3])                                // Callstack
   AADD (option, optlist[4])                                // Debug
   color := if (iscolor(), "W+/BG,W+/R", NIL)
else
   AADD (option, optlist[1])                                // Abort
   IF VALTYPE(e_obj) == "O" .and. e_obj:SEVERITY != ES_FTLERR
      AADD (option, optlist[2])                             // Ignore
      AADD (option, optlist[3])                             // Callstack
      AADD (option, optlist[4])                             // Debug
   ELSE
      AADD (option, optlist[3])                             // Callstack
   ENDIF
endif

DISPEND()

DO WHILE .T.
   IF choice == 0
      IF replopt > 0
         option[replopt] := optlist[3]              // Callstack
         replopt := 0
      ENDIF
      choice := __ERRALERT(msg_str, option, color)  // Display menu
   ENDIF
   IF choice == 0
      EXIT
   ENDIF
   cc  := option[choice]
   old := choice

   FOR choice := 1 TO LEN(optlist)           // choice = index in optlist[]
      IF optlist[choice] == cc
         EXIT
      ENDIF
   NEXT

   DO CASE
   CASE choice == 2                                 // Ignore
      EXIT
   CASE choice == 3                                 // Callstack
      msg_str2 := ""
      ii := START_STACK
      WHILE (LEN(prc := PROCNAME(++ii)) != 0)
        IF LEN(msg_str2) != 0
          msg_str2 += ";"
         ENDIF
         msg_str2 += PADR(PROCFILE(ii), 10) +  STR(PROCLINE(ii), 5) + ;
               ":" + PADR(prc + "()", 12)
      ENDDO
      option[old] := optlist[6]                     // Errormessage
      choice := __ERRALERT(msg_str2, option, color) // Display it
      IF choice == old
         choice  := 0                               // go back
         replopt := old
      ENDIF
      LOOP

   CASE choice == 4                                 // Debugger
#Cinline
      __sig_quit |= 1;    /* Call debugger as soon as we are out of err.sys */
      _sstep |= 1;
#endCinline
      EXIT

   CASE choice == 5                                 // Print
#ifdef INCLUDE_PRINT
      sav_print := SET (_SET_PRINTER)
      sav_cons  := SET (_SET_CONSOLE)
      SET CONSOLE OFF
      SET PRINTER ON
      ?
      ? "---Error Callstack and Print screen---"
      AEVAL (__splittxt(msg_str), {|x| QOUT(x)} )
      ?
      ii := START_STACK
      WHILE (LEN(prc := PROCNAME(++ii)) != 0)
         ? PADR(PROCFILE(ii), 10) +  STR(PROCLINE(ii), 5) + ;
           ":" + PADR(prc + "()", 12)
      ENDDO
      ?
      err_print()
      SET (_SET_PRINTER, sav_print)
      SET (_SET_CONSOLE, sav_cons )
      ADEL  (option, 1)                             // remove print option
      ASIZE (option, LEN(option) -1)
      choice := 0
      LOOP
#else
      EXIT
#endif

   OTHERWISE
      QUIT

   ENDCASE
ENDDO

DEVPOS(old_row, old_col)                            // reset old cursor pos.
PUBLIC FS_LAST_ERR := e_obj
RETURN NIL

/*------------------------*
 | optional: print screen |
 *------------------------*/
#ifdef INCLUDE_PRINT
   STATIC FUNCTION err_print ()
   LOCAL_INT ymax := MAXROW()                       // used also in C
   LOCAL     chr_arr[ymax +1]                       // e.g. chr_arr[25]
   LOCAL     ii := 0


#Cinline
   {
#include <malloc.h>
      fsFnPROTOT(savescreen);                       /* if not used yet */
      fsFnPROTOT(maxcol);                           /* if not used yet */

      FSvar *actscreen = VAR_NEW;
      WINDOW   *winPtr;
      int      x, y, xmax;
      unsigned char *oneline, *str;

      actscreen = fsFnNAME(savescreen)(0,0);               /* SAVE SCREEN   */
      winPtr    = VAR_SCR (actscreen);
      xmax      = (int) VAR_NUM (fsFnNAME(maxcol)(0,0) );  /* xmax=MAXCOL() */

      oneline = malloc( (unsigned) ((xmax +1) * sizeof(unsigned char)));
      if (oneline != NULL) {
         for (y=0; y <= ymax; y++) {
            str = oneline;
            for (x=0; x <= xmax; x++)
               *(str++) = (unsigned char)
                          (mvwinch(winPtr,y,x) & A_CHARTEXT);    /* curses */
            SET_VAR_CHRLEN (VAR_ARRELEM (VAR_NAME_LOCAL(chr_arr), y+1),
                            oneline, xmax +1);
         }
         free (oneline);
      }
   }
#endCinline
   ii := 0
   ? SPACE(4) + REPLICATE("---------+", 8)
   AEVAL (chr_arr, {|x| QOUT(STR(ii++,2) + ":", x)} )
   ? SPACE(4) + REPLICATE("---------+", 8)
   ?
   RETURN NIL
#endif

*** eof ***

