/****************************************************************************
 * fserrors.h    header file for fserrmsg.c and fserror.c                   *
 * -------------------------------------------------------------------------*
 * Copyright (c) 1998 multisoft Datentechnik GmbH. All rights reserved.     *
 ****************************************************************************/

struct __error_report
{
    char *err_msg;                  /* ...pointer to error message          */
    int when;                       /* ...indicates at what                 */
                                    /* fs_set_flags[program_phase] this     */
                                    /* error should be reported             */
};

extern WINDOW *ERR_WIND, *ERR_FRAME;

extern char *fgs_errmsg[];

#define GENRAL       0        /* general (unspecified) error           */

#define IOOPEN       101      /* file open error                       */
#define IOCLOS       102      /* file close error                      */
#define IOREAD       103      /* file read error                       */
#define IOWRIT       104      /* file write error                      */
#define IOCREAT      105      /* file write error                      */
#define IOERASE      106      /* file write error                      */
#define IOSTAT       107      /* file status error                     */

#define RECNOOR      1        /* record number out of range            */

#define DTMMACH      201      /* data type mismatch                    */
#define DTILOPR      202      /* illegal operation on data type        */
#define DTILEGL      203      /* illegal data type                     */
#define DTILLFL      204      /* illegal field type                    */
#define DTILVAL      205      /* illegal field value                   */

#define RTNOMEM      301      /* no memory available                   */
#define RTNOVAR      302      /* no memory variables available         */
#define RTNLMVP      303      /* null pointer for memory variable      */
#define RTXPMPT      304      /* too many prompts for menu             */
#define RTMXGET      305      /* too many gets                         */
#define RTNOFLD      306      /* cannot find that field                */
#define RTUNVAR      307      /* undefined variable                    */
#define RTUNFUN      308      /* undefined function                    */
#define RTSYNTX      309      /* Syntax error in expression evaluator  */
#define RTEEVAL      310      /* Run-time system yields NULL pointer   */
#define RTARGCNT     311      /* Wrong argument count                  */
#define RTARGTYP     312      /* Wrong argument type                   */
#define RTARGVAL     313      /* Wrong argument value                  */
#define RTENDLMEXP   314      /* Endless loop in macro expansion       */
#define RTTLMEXP     315      /* Too long macro expansion              */
#define RTMISARG     316      /* Missing argument                      */
#define RTILLTYP     317      /* Illegal type                          */
#define RTIVSKEL     318      /* Illegal skeleton                      */
#define RTSTRTL      319      /* String too long for runtime system    */
#define RTILLVAL     320      /* Illegal value                         */
#define RTOSERR      321      /* Operating system command returned err */
#define RTNODBF      322      /* No database opened                    */
#define RTFILNAM     323      /* Wrong file name                       */
#define RTARGNLS     324      /* arg is null string                    */
#define RTNOLOCK     325      /* DBF not locked (and should be)        */
#define RTDBFVER     326      /* wrong DBF version                     */
#define RTNOMORLCKS  327      /* Lock count for dbf exclusive exdceeded*/
#define RTWRNGLIN    328      /* reading a line in the scrmap defin    */
#define RTILLCOL     329      /* Illegal color specification           */
#define RTNODBF1     330      /* no dbf open, but only when development*/
#define RTIDXCRPT    331      /* Index file corrupted                  */
#define RTCONVAR     332      /* constant variable                     */
#define RTILLPICT    333      /* wrong picture in say or get           */
#define RTILLEOF     334      /* DBF on eof or bof but record needed   */
#define RTILLBRK     335      /* Illegal break                         */
#define RTILLDATE    336      /* wrong dateformat in set()             */
#define RTILLNAM     337      /* undefined identifier                  */
#define RTILLCLS     338      /* Illegal object class                  */
#define RTUSERS      339      /* unable to open file                   */
#define RTIDXVER     340      /* wrong IDX version                     */
#define RTNOFLD1     341      /* cannot find that field                */
#define RTILLALIAS   342      /* Illegal Alias                         */
#define RTILLMET     343      /* Illegal Method                        */
#define RTILLINS     344      /* Illegal Instance                      */
#define RTLOCKFAIL   347      /* Lock failed                           */

#define INTARGCN     401      /* argcnt negative                       */
#define INTARGCB     402      /* too many args                         */
#define INTARGCL     403      /* too little args                       */
#define INTARGTYP    404      /* wrong arg type                        */
#define INTARGNEG    405      /* arg negative                          */
#define INTARGNUL    406      /* arg null                              */
#define INTARGNLP    407      /* arg null pointer                      */
#define INTARGNLS    408      /* arg points to null string             */
#define INTARGOR     409      /* arg out of range                      */
#define INTCMDFLG    410      /* flag has impossible value             */
#define INTMEMOUT    411      /* out of memory                         */
#define INTNDXSEC    412      /* non-existing index data sector access */
#define INTDBFNO     413      /* database file (DBF) not opened        */
#define INTDBTNE     414      /* DBF file has no memo file (DBT)       */
#define INTNDXNO     415      /* database index file (IDX) not opened  */
#define INTILLVAL    416      /* illegal internal value                */

#define FTNOTOPN     501      /* unable to open file                   */
#define FTFUNARG     502      /* too many function arguments           */

#define LOCKNDX      701      /* Index file already locked             */

#define GENWARNING   901      /* General warning                       */
#define DEVWARNING   902      /* Develop warning                       */

#define MAXERRMSG    999      /* maximum error message number          */


/* report error at time ...                                            */

#define RP_ALWAYS    1000     /* always                                */
#define RP_INUSE     800
#define RP_INDEVELOP 700      /* only if FS_SET ("devel", .T.) is set  */
#define RP_NEVER     0        /* never                                 */


/* error types codes                                                    */

#define ERRTYP_IO   1       /* input-output error type                  */
#define ERRTYP_RT   2       /* run-time error type                      */
#define ERRTYP_FTL  4       /* fatal error type                         */
#define ERRTYP_INT  8       /* internal error type                      */
#define ERRTYP_DUM  16      /* internal error type                      */

extern int fs_set_flag[];

#define IS_DEVELOP_MODE (fs_set_flag[0] == RP_INDEVELOP ? 1 : 0)

/**** EOF ****/

