*-----------------------------------------------------------------------------
* autolock.prg (c) 1998 by multisoft Datentechnik GmbH, Munich, Germany
*
* The following AUTORLOCK(), AUTOFLOCK() and AUTOUNLOCK() functions are
* invoked automatically from the (default) RDD driver DBFIDX in SHARED mode
* before write attempt, when no user RLOCK() or FLOCK() was performed and
* SET AUTOLOCK is currently active (the default) or the oRdd:CONCURRENCY-
* CONTROL instance is set. See description in the manual.
*
* These functions may be used for both procedural invocation (w/o argument)
* optionally prefaced with alias, and for Object oriented invocation (pass
* the RDD object as an argument).
*
* If METHODs named AutoRlock(), AutoFlock(), and AutoUnock() are available
* in the currently used RDD, these methods are preferred.
*
* These functions are available in object form in the FlagShip library.
* You may freely modify this file for your own purposes, e.g. modify the
* default BREAK invocation, activate the log protocol etc. But it is not
* recommended to change the general behaviour, since these functions ARE
* called from the default RDD and probably other drivers.
* It is recommended to modify a local copy of this file, not the original
* one stored in the /usr/FSsrc/system directory.
*
* Compile: FlagShip -na -c autolock.prg
*      or: FlagShip -na -c -DPROTOCOL_ON autolock.prg
* Link   : FlagShip myappl*.prg autolock.o ....
*
*-----------------------------------------------------------------------------

#include "set.fh"
#include "rddsys.fh"

* #define PROTOCOL_ON                    /* enable, if required           */
#define   WITH_BREAK                     /* enable, if required           */
#define   RDD_INFO_AVAILABLE             /* RDD specific                  */
#define   PREFER_RDD_METHOD              /* disable, if method in RDD     */
#define   RETRY_PERIOD 1                 /* seconds between retry > 0.1 ! */

#define OUT_MSG1 {"Try again", "Break to menu", "Ignore", "Quit"}
#define OUT_MSG2 "Trying autoRLOCK() for "
#define OUT_MSG3 "Cannot perform autoRLOCK() for "
#define OUT_MSG4 "Trying autoFLOCK() for "
#define OUT_MSG5 "Cannot perform autoFLOCK() for "
#define OUT_MSG6 "Trying autoUNLOCK for "
#define OUT_MSG7 "Cannot perform autoUNLOCK for "
#define OUT_MSG8 "Auto record lock"
#define OUT_MSG9 "Auto file lock"

******************************************************************************
* AUTORLOCK([oRDD], [nWait]) -> retL
*
* "smart" RLOCK()
*
FUNCTION AUTORLOCK (oRDD, nWait) AS LOGICAL
#ifdef PROTOCOL_ON
   outProtocol("autoRlock() ")           // logg protocol
#endif
return AutoWaitProc (oRDD, nWait, 1, OUT_MSG2, OUT_MSG3, OUT_MSG8)

******************************************************************************
* AUTOFLOCK([oRDD], [nWait]) -> retL
*
* "smart" FLOCK()
*
FUNCTION AUTOFLOCK (oRDD, nWait) AS LOGICAL
#ifdef PROTOCOL_ON
   outProtocol("autoFlock() ")           // logg protocol
#endif
return AutoWaitProc (oRDD, nWait, 2, OUT_MSG4, OUT_MSG5, OUT_MSG9)

******************************************************************************
* AUTOUNLOCK([oRDD], [nWait]) -> retL
*
* "smart" UNLOCK
*
FUNCTION AUTOUNLOCK (oRDD, nWait) AS LOGICAL
#ifdef PROTOCOL_ON
   outProtocol("autoUnlock()")           // logg protocol
#endif
if valtype(oRDD) == "O"
   oRDD:COMMIT()                         // commit current database
else
   DBCOMMIT()                            // commit current database
endif
return AutoWaitProc (oRDD, nWait, 3, OUT_MSG6, OUT_MSG7, "")

******************************************************************************
* static (local working) functions follow
*
STATIC FUNCTION AutoWaitProc (oRDD, nWait, iType, cTxt1, cTxt2, cTxt3)
local nTime  := SECONDSCPU(4)
local lockOk := .F., isMsg := .F., nUserIn := 0
local isObject := (valtype(oRDD) == "O")
local cFullName:= "", cFunctName := procname(1)

if nWait == NIL
   nWait  := SET(_SET_AUTOLOCK)
endif

if nWait < -2                            // see SET AUTOLOCK
   return .F.                            // AutoLock not active
elseif nWait <= 0
   nWait := 99999999
endif
lockOk := if (isObject, oRDD:used, used())
if !lockOk
   return .F.
endif

#ifdef PREFER_RDD_METHOD
   if !isobject                          // If the same named Method
      oRdd := dbobject()                 // is available in RDD,
   endif                                 // use it preferrably
   if isobjproperty(oRdd, 4, cFunctName) 
      return oRdd:&cFunctName()          // depends on the RDD functionality
   endif
#endif

while .T.                                // try again ?
   while .T.                             // loop for the lock retry
      do case
      case iType == 1
         if isObject
            lockOk := oRDD:RLOCK()       // object
         else
            lockOk := RLOCK()            // std. RLOCK()
         endif
      case iType == 2
         if isObject
            lockOk := oRDD:FLOCK()       // object
         else
            lockOk := FLOCK()            // std. FLOCK()
         endif
      otherwise
         if isObject
            lockOk := oRDD:UNLOCK()      // object
         else
            UNLOCK                       // std. UNLOCK
            lockOk := .T.
         endif
      endcase
      if lockOk
         exit                            // successful
      endif

      if empty(cFullName)
#ifdef RDD_INFO_AVAILABLE
         if !isObject
            oRDD := DBOBJECT()           // get the RDD object
         endif
         cFullName := oRDD:INFO(DBI_FULLPATH)
         if empty(cFullName)
            cFullName := DBF()
         endif
#else
         cFullName := DBF()              // name of the database
#endif
      endif

      if (SECONDSCPU(4) - nTime) > nWait // SET AUTOLOCK time expired ?
         lockOk := .F.                   // yes
         exit
      endif
      if !isMsg
         isMsg := outMessage(cTxt1 + cFullName) // display "trying.." message
      endif
      inkey (RETRY_PERIOD)               // wait
   enddo                                 // end-of-loop for the lock retry

   if isMsg                              // clear "trying.." message
      isMsg := outMessage()
   endif
   if lockOk                             // successful ?
      exit
   endif

   nUserIn := userAction(cTxt2 + cFullName, OUT_MSG1)  // user choice
   if nUserIn == 1                       // try again
      nTime := SECONDSCPU(4)
      loop
   elseif nUserIn == 3                   // ignore
      exit
   elseif nUserIn == 4                   // quit
      quit
   elseif nUserIn == 2                   // break
#ifdef WITH_BREAK
      if ISBEGSEQ()                      // Begin Sequence ... active?
         BREAK cTxt3                     // yes, jump to the
      endif                              // next RECOVER or ENDSEQUENCE
#endif
   endif
   ?? chr(7)
enddo
return lockOk

#ifdef PROTOCOL_ON
STATIC FUNCTION outProtocol (cTxt)
  local is_cons := SET(_SET_CONSOLE)
  local is_prin := SET(_SET_PRINTER)
  set console off
  set printer on
  ? cTxt + " in " + procname(2) + "/" + ltrim(str(procline(2))), date(),time()
  set printer (is_prin)
  set console (is_cons)
  return NIL
#endif

STATIC FUNCTION outMessage (cTxt)
static r1, r2, c1, c2, scr
local  color := SetColor()
if empty(cTxt)
   if valtype(scr) $ "SC"
      restscreen (r1,c1,r2,c2,scr)
   endif
   return .F.
endif
r1 := int((maxrow() - 5) / 2)
r2 := r1 + 4
c1 := int((maxcol() - len(cTxt) - 4) / 2)
c2 := c1 + len(cTxt) + 3
scr:= savescreen (r1,c1,r2,c2)
setcolor (if(iscolor(), "GR+/R", "/W"))
@ r1,c1 clear to r2,c2
@ r1,c1 to r2,c2
@ r1 + 2, c1 + 2 say cTxt
setcolor (color)
return .T.

STATIC FUNCTION userAction (cTxt, aResponse)
return alert (cTxt, aResponse)

*** eof ***
