** BENCH1FX.PRG ** Sieve of Erotosthenes, using numeric array or string

** Notes:
** This is a modified program bench1.prg for FoxPro. You will probably
**   need to modify the SET DEFAULT and SET PROCEDURE statements
** With FlagShip, use bench1.prg

SET DEFAULT   to c:\tmp
SET PROCEDURE to c:\tmp\bench1f_.prg

private i, j, k
dimension flags(500)
private sflags
private duration, start
private iters, ccount, prime

#define ITERATIONS 500
#define SIZE       500

iters = ITERATIONS
count = 0
prime = 0

* CLEAR

* #ifndef FlagShip
*    ? 'Clipper 5.x (Locals)'
* #else
*    ? 'FlagShip (Locals + Typed)'
* #endif
?  "FoxPro"
?? " Sieve of Erotosthenes,", ltrim(str(ITERATIONS)), "iterations"

? "1. Test for numeric calculations"
************************************
start = SECONDSCPU()

FOR k = 1 TO ITERATIONS
   count = 0
   FOR i = 1 TO SIZE
      flags[i] = 1
   endfor					   && NEXT
   FOR i = 1 TO SIZE
      IF flags[i] != 0
         prime = 2 * i + 3
         j     = i + prime
         do WHILE j <= SIZE
            flags[j] = 0
            j = j + prime      && j += prime
         ENDDO
         count = count + 1     && count++
      ENDIF
   endfor					   && NEXT
endfor					       && NEXT

duration = SECONDSCPU() - start
? '   Number of primes were '
?? LTRIM(STR(count)) + ".  "
?? LTRIM(STR(iters)), 'iterations took '
   do dispout with transform(duration, '999.99'), "W+"
?? ' seconds'

? "2. Test for string handling"
*********************************
sflags = replicate(CHR(1), SIZE)

start = SECONDSCPU()

FOR k = 1 TO ITERATIONS
   count = 0
   FOR i = 1 TO SIZE
      IF STRPEEK (@sflags,i) != 2
         prime = 2 * i + 3
         j     = i + prime
         do WHILE j <= SIZE
            sflags = STRPOKE (sflags, j, 2)  && STRPOKE (@sflags, j, 2)
            j = j + prime           && j += prime
         ENDDO
         count = count + 1          && count++
      ENDIF
   endfor					        && NEXT
endfor					            && NEXT

duration = SECONDSCPU() - start
? '   Number of primes were '
?? LTRIM(STR(count)) + ".  "
?? LTRIM(STR(iters)), 'iterations took '
   do dispout with transform(duration, '999.99'), "W+"
?? ' seconds'

* #ifdef FlagShip
* ? "3. Test for C-inline usage"
* *****************************
* start = SECONDSCPU()
* 
* FOR k = 1 TO ITERATIONS
*    count = 0
* #Cinline
*    for (i = 0; i <= SIZEC; i++)
*       cflags[i] = 1;
* 
*    for (i = 0; i <= SIZEC; i++) {
*       if (cflags[i]) {
*          prime = 2 * i + 3;
*          j     = i + prime;
*          while (j <= SIZEC) {
*             cflags[j] = 0;
*             j += prime;
*          }
*          count++;
*       }
*    }
* #endCinline
* NEXT
* 
* duration = SECONDSCPU() - start
* ? '   Number of primes were '
* ?? LTRIM(STR(count)) + ".  "
* ?? LTRIM(STR(iters)), 'iterations took '
*    dispout(transform(duration, '999.99'), "W+")
* ?? ' seconds'
* #endif

* wait

?
? "Guideline values for identical HW :"
?
set color to "W+"
? "a) PP200, 64MB, Win-NT4 vs. Linux or SCO :"
? "                                         Clipper 5.3  FlagShip 4.4   "
do DISPOUT with "faster", "GR+"
set color to "W"
? "1. numer.calculation,   500 interations     9.61 sec    1.27 sec  = "
do DISPOUT with "  7.6 x", "GR+"
? "2. string manipulation, 500 interations    76.35 sec    2.51 sec  = "
do DISPOUT with " 30.4 x", "GR+"
? "3. num.calc, C-inline,  500 interations     9.61 sec    0.07 sec  = "
do DISPOUT with "137.2 x", "GR+"
?
set color to "W+"
? "b) PP200, 64MB, Win-NT4 vs. Linux or SCO :"
? "                                          FoxPro 2.6  FlagShip 4.4   "
do DISPOUT with "faster", "GR+"
set color to "W"
? "1. numer.calculation,   500 interations     4.58 sec    1.27 sec  = "
do DISPOUT with "  3.6 x", "GR+"
? "2. string manipulation, 500 interations    45.36 sec    2.51 sec  = "
do DISPOUT with " 18.1 x", "GR+"
? "3. num.calc, C-inline,  500 interations     4.58 sec    0.07 sec  = "
do DISPOUT with " 65.4 x", "GR+"
?
** #ifdef FlagShip
** if usersmax() == 4
**   ?
**   dispout("Note:", "r+") ; qqout(" the used FlagShip DEMO version is ")
**   dispout("slower", "R+"); qqout(" than the licensed version,")
**   ? "      so don't use it for true benchmarks "
**   dispout(":-)", "R+")
** endif
** #endif

wait
quit

** The functions are available in bench1p.prg
** #ifndef FlagShip
*  FUNCTION strpeek 
*  parameters flags, pos
*  RETURN ASC( SUBSTR(flags, pos, 1))
*
*  FUNCTION strpoke 
*  parameters flags, pos, val
*  flags = STUFF (flags, pos, 1, CHR(val))
*  RETURN flags
*
*  FUNCTION DISPOUT 
*  parameters text, color
*  @ row(), col() SAY text COLOR &color
*  RETURN
** #endif

** eof **

