*** Benchmark Clipper 5.x vs. FlagShip
***
*** Compile: FlagShip bench.prg -obench ; ./bench
***      or: Clipper  bench.prg ; Blinker fi bench ; bench

// Enable (uncomment) the following #define line if you want to display the 
// overall time, i.e. the user+system and user+system+wait times. In this
// case, you will need the terminal set to approx. 85 columns. The time 
// difference is notable when your data are accessed remotely, e.g. via NFS
*  #define DISPLAY_FULL
*  SET DEFAULT TO /my/mount/point

PARAMETERS cmd1                          // bench <filename>

#ifdef FlagShip
   FS_SET("lower", .T.)                  // convert file names to lowercase
*  FS_SET("pathlower", .T.)              // convert path names to lowercase
   FS_SET("translext", "ntx", "idx")     // search .idx instead of .NTX
#  define SECONDS SECONDSCPU
#else
#  define LOCAL_LONG   LOCAL
#  define LOCAL_DOUBLE LOCAL
#  define STATIC_LONG  STATIC
#endif

private RECORDS := 10000
private RECORD2 := 1000

#ifndef FlagShip
FlagShip := .F.
#endif

private isEval := .F.
private EXCL   := .T.
private SHARE  := .F.

private txtBuff := {}
private times1  := {{5.11,0.03}, {6.64,0.48}, {6.65,0.06}, ;
                    {6.42,0.46}, {6.39,0.05}, {9.61,0.37}, ;
                    {9.61,0.09}, {8.74,0.33}, {8.68,0.09}, ;
                    {8.95,0.37}, {8.91,0.11} }
private times2  := {{ 0.02, 0.04}, {10.24, 0.35}, {67.22, 1.76}, ;
                    {43.56, 7.24}, {11.92, 1.95}, {10.82, 1.96}, ;
                    { 3.57, 0.46}, { 3.18, 0.43}, {16.71, 0.98}, ;
                    {21.23, 1.09}, {21.29, 5.41}, {22.46, 0.51}, ;
                    {22.58, 0.71}, {27.18, 0.71} }

col1 := IF (ISCOLOR(), "W+/B,N/W", "W/N,/W")
col2 := IF (ISCOLOR(), "GR+/B", "W+/N")
dbfname = "benchmark"
index1  = "bench1"
index2  = "bench2"

print = .F.
if PCOUNT() > 0
   if !empty(cmd1)
      print := .T.
      ?
      ? "Printer output to file " + cmd1
      text = ""
      accept "Additional text:" to text
      SET PRINTER TO (cmd1)
      while !empty(text)
         SET CONSOLE OFF
         SET PRINTER ON
         ? text
         SET PRINTER OFF
         SET CONSOLE ON
         text = ""
         accept "Additional text:" to text
      enddo
      SET CONSOLE OFF
   endif
endif

SET COLOR TO &col1
clear screen

#ifdef FlagShip
if usersmax() == 4
  ?
  ? "Note, this is a DEMO version, which allows to write 100 records only."
  ? "The benchmark on 100 records will therefore be less representative."
  wait
  clear screen
  RECORDS := 100
  RECORD2 := 10
  isEval := .T.
  SHARE  := .T.   // excl required for PACK, ZAP

endif
#endif

center (1, VERSION() + " Benchmark", col2)
line = 3
set console off

dbf_crea  (line++, " 1.Create database")
my_use (dbfname, EXCL)
dbf_app1  (line++, " 2.Append     " + ltrim(RECORDS) + " records, exclusive", RECORDS)
* dbf_zap   (line++)
ZAP
my_use (dbfname, SHARE)
dbf_app1  (line++, " 3.Append     " + ltrim(RECORDS) + " records, shared", RECORDS)
dbf_repl  (line++, " 4.Replace 2x " + ltrim(RECORDS) + " records, shared", 0, RECORDS)

* my_use (dbfname, EXCL)
dbf_ind1  (line++, " 5.Index      " + ltrim(RECORDS) + " records on character")
dbf_ind2  (line++, " 6.Index      " + ltrim(RECORDS) + " records on number")

my_use (dbfname, SHARE)
SET INDEX TO (index1), (index2)
dbf_seek1 (line++, " 7.Seek       " + ltrim(RECORD2) + "x shared on character")
SET ORDER TO 2
dbf_seek2 (line++, " 8.Seek       " + ltrim(RECORD2) + "x shared on numeric")

dbf_repl  (line++, " 9.Replace 2x " + ltrim(RECORD2) + "x records, shared w. 2 indices", 0, RECORD2)
dbf_repl  (line++, "10.Replace 2x " + ltrim(RECORD2) + "x records, shared w. 2 indices & DbCommit()", 1, RECORD2)
dbf_repl  (line++, "11.Replace 2x " + ltrim(RECORD2) + "x records, shared w. 2 indices & COMMIT ", 2, RECORD2)
if isEval
   delete all
   pack
endif
dbf_app1  (line++, "12.Append     " + ltrim(RECORD2) + "x records, shared w. 2 indices", RECORD2)
if isEval
   delete all
   pack
endif
dbf_app3  (line++, "13.Append     " + ltrim(RECORD2) + "x records, shared w. 2 indices & SKIP 0", RECORD2)
if isEval
   delete all
   pack
endif
dbf_app2  (line++, "14.Append     " + ltrim(RECORD2) + "x records, shared w. 2 indices & DbCommit()", RECORD2)

set console on
?
if isEval
   cColor := setColor ("GR+/B")
   ? " Note, the above DEMO results are not representative, since calculated on 100"
   ? " records only and since the Demo version is slower than the full release."
   setColor (cColor)
endif
wait
?
? "Guideline values for identical HW (PP200, 64MB)"
? "Clipper 5.3 on DOS/Win-NT vs. FlagShip 4.42           Clipper    FS  = faster"
?
for ii := 1 to len(txtbuff)
   ? substr ((txtBuff[ii] + " " + repli(".", 52)), 1, 52) + ;
     str(times2[ii,1],6,2) + " vs." + str(times2[ii,2],6,2) + " = "
   xcolor := setcolor("GB+/B")
   ?? str(times2[ii,1] / times2[ii,2], 5,1) 
   setcolor(xcolor)
   ?? " x"
next
?
wait

txtBuff := {}
cls
center (1, VERSION() + " Benchmark", col2)
line = 3

test1 (line++, " 1.Empty loop     200,000 x",              0)
test1 (line++, " 2.Addition       200,000 x integer"  ,   10)
test1 (line++, " 3.Addition       200,000 x typed int",   11)
test1 (line++, " 4.Increment      200,000 x integer",     20)
test1 (line++, " 5.Increment      200,000 x typed int",   21)
test1 (line++, " 6.Multiplication 200,000 x integer"  ,   30)
test1 (line++, " 7.Multiplication 200,000 x typed int",   31)
test1 (line++, " 8.Multiplication 200,000 x float",       32)
test1 (line++, " 9.Multiplication 200,000 x typed float", 33)
test1 (line++, "10.Division       200,000 x float",       40)
test1 (line++, "11.Division       200,000 x typed float", 41)

line++
if print
   set printer on
   ?
   set printer off
endif

set console on
?
wait
?
? "Guideline values for identical HW (PP200, 64MB)"
? "Clipper 5.3 on DOS/Win-NT vs. FlagShip 4.42           Clipper    FS  = faster"
?
for ii := 1 to len(txtbuff)
   ? substr ((txtBuff[ii] + " " + repli(".", 52)), 1, 52) + ;
     str(times1[ii,1],6,2) + " vs." + str(times1[ii,2],6,2) + " = "
   xcolor := setcolor("GB+/B")
   ?? str(times1[ii,1] / times1[ii,2], 5,1) 
   setcolor(xcolor)
   ?? " x"
next
? 
if isEval
   ?
   xcolor := setcolor("r+/b") ; qqout("Note:") ; setcolor(xcolor)
   qqout(" the FlagShip DEMO version is ")
   xcolor := setcolor("r+/b") ; qqout("slower") ; setcolor(xcolor)
   qqout(" than the licensed version,")
   ? "      so don't use it for true benchmarks "
   xcolor := setcolor("r+/b") ; qqout(":-)") ; setcolor(xcolor)
   ?
endif
wait

set printer to
quit

***************************************************************************
FUNCTION my_use (dbfname, exclus)
IF exclus
   USE (dbfname) EXCLUSIVE
ELSE
   USE (dbfname) SHARED
ENDIF
DO WHILE NETERR()
   @ 0,0 say "waiting for USE " + dbfname + ".dbf, " + ;
         IF (exclus, "EXCLUSIVE", "SHARED")
   inkey(2)
   if lastkey() = 27
      QUIT
   endif
   IF exclus
      USE (dbfname) EXCLUSIVE
   ELSE
      USE (dbfname) SHARED
   ENDIF
   @ 0,0
ENDDO
RETURN NIL

***************************************************************************
FUNCTION test1 (line, text, chk)
STATIC loop_time, string
LOCAL  act_time, temp
LOCAL_LONG   ii
LOCAL_LONG   sumLong  := 1
LOCAL_DOUBLE sumFloat := 1.0
LOCAL sumIntVar       := 1
LOCAL sumNumVar       := 1.0

output (line, text)
act_time = SECONDS()

DO CASE
CASE chk = 0
   FOR ii := 1 TO 200000
   NEXT
CASE chk = 10
   FOR ii := 1 TO 200000
      sumIntVar := sumIntVar + 1
   NEXT
CASE chk = 11
   FOR ii := 1 TO 200000
      sumLong := sumLong + 1
   NEXT
CASE chk = 20
   sumIntVar = 1
   FOR ii := 1 TO 200000
      sumIntVar++
   NEXT
CASE chk = 21
   sumLong = 1
   FOR ii := 1 TO 200000
      sumLong++
   NEXT
CASE chk = 30
   FOR ii := 1 TO 200000
      sumIntVar := 1
      sumIntVar *= 2
   NEXT
CASE chk = 31
   FOR ii := 1 TO 200000
      sumLong := 1
      sumLong := sumLong * 2
   NEXT
CASE chk = 32
   FOR ii := 1 TO 200000
      sumNumVar := 1.0
      sumNumVar *= 1.123
   NEXT
CASE chk = 33
   FOR ii := 1 TO 200000
      sumFloat := 1.0
      sumFloat := sumFloat * 1.123
   NEXT
CASE chk = 40
   FOR ii := 1 TO 200000
      sumNumVar := 30000.0
      sumNumVar /=  1.005
   NEXT
CASE chk = 41
   FOR ii := 1 TO 200000
      sumFloat := 30000.0
      sumFloat := sumFloat / 1.005
   NEXT
ENDCASE
RETURN output (SECONDS() - act_time)


***************************************************************************
FUNCTION dbf_crea (line, text)
LOCAL stru := {{"char", "C", 10, 0}, ;
               {"num",  "N", 10, 0}}
output (line, text)
DBCREATE (dbfname, stru)
RETURN output ()

***************************************************************************
FUNCTION dbf_app1 (line, text, count)
LOCAL_LONG ii
output (line, text)
FOR ii := 1 to count
   APPEND BLANK
NEXT
COMMIT
RETURN output ()

***************************************************************************
FUNCTION dbf_app2 (line, text, count)
LOCAL_LONG ii
output (line, text)
FOR ii := 1 to count
   APPEND BLANK
   DbCommit()
NEXT
RETURN output ()

***************************************************************************
FUNCTION dbf_app3 (line, text, count)
LOCAL_LONG ii
output (line, text)
FOR ii := 1 to count
   APPEND BLANK
   SKIP 0
NEXT
RETURN output ()

***************************************************************************
FUNCTION dbf_zap  (line, text)
output (line, text)
ZAP
RETURN output ()

***************************************************************************
FUNCTION dbf_repl (line, text, commit, count)
LOCAL first := "z", rec := 1
output (line, text)
GOTO TOP
DO WHILE !EOF() .and. rec <= count
   DO WHILE !RLOCK()
      @ 0,0 say "waiting for RLOCK"
      if INKEY(1) == 27
         QUIT
      ENDIF
      @ 0,0
   ENDDO
   REPLACE num  WITH RECNO() + 1
   REPLACE char WITH first + " text"
   first := IF (first == "A", "z", CHR(ASC(first) -1))
   UNLOCK
   do case
   case commit == 1
      DbCommit()
   case commit == 2
      COMMIT
   endcase
   SKIP
   rec++
ENDDO
RETURN output ()

***************************************************************************
FUNCTION dbf_ind1 (line, text)
output (line, text)
INDEX ON char to (index1)
RETURN output ()

***************************************************************************
FUNCTION dbf_ind2 (line, text)
output (line, text)
INDEX ON num to (index2)
RETURN output ()

***************************************************************************
FUNCTION dbf_seek1 (line, text)
LOCAL_LONG ii
LOCAL first, random, count := 0, col
output (line, text)
FOR ii := 1 to (if (isEval, 100, RECORD2))
   random := ii % 56
   first = CHR(random + 65)
   SEEK first + " text"
   IF !FOUND() .and. val(first) > 0
      @ 0,0 SAY "[" + first + " text] not found (" + LTRIM(STR(++count)) + ")"
      col := COL()
   ENDIF
NEXT
output ()
IF count > 0
   @ 0, col say "...any key..."
   INKEY(0)
   @ 0,0
ENDIF
RETURN NIL

***************************************************************************
FUNCTION dbf_seek2 (line, text)
LOCAL_LONG ii
LOCAL first := 4999, count := 0
LOCAL col
if isEval
   first := 99
endif
output (line, text)
FOR ii := 1 to (if (isEval, 99, RECORD2))
   SEEK (first--)
   IF !FOUND() .and. first > 0
      @ 0,0 SAY "[" + STR(first,4) + "] not found (" + LTRIM(STR(++count)) + ")"
      col := COL()
   ENDIF
NEXT
output ()
IF count > 0
   @ 0, col say "...any key..."
   INKEY(0)
   @ 0,0
ENDIF
RETURN NIL

***************************************************************************
FUNCTION center (line, text, color)
@ line, (min(79,MAXCOL()) - LEN(text)) / 2 SAY text COLOR color
if print
   set printer on
   ?
   ? text + " at "
   ?? date(), time()
   ?
   set printer off
endif
RETURN NIL

***************************************************************************
FUNCTION output (line, text)
STATIC time, time_sec, row, col, pcol
LOCAL ii, loctime
IF PCOUNT() > 1
   aadd (txtBuff, text)
   if print
      set printer on
      ? text
      pcol := PCOL()
      set printer off
   endif
   if line > MAXROW()
      scroll (0,0, MAXROW(), MAXCOL(), 1)
      line = MAXROW()
   endif
   @ line,0 say text
   time := SECONDS()
   time_sec := seconds()
   row  := ROW()
   col  := COL()
ELSE
   @ row, col+1 say ""
   SET CONSOLE ON
   FOR ii = col +1 TO 61
      ?? "."
   NEXT
   loctime := IF (PCOUNT() > 0, line, SECONDS() - time)
   @ row, 63 SAY loctime PICT "9999.99" COLOR col2
#ifdef DISPLAY_FULL
   ii := seconds() - time_sec
   ?? " =" , str(ii, max(2, len(ltrim(ii))), 0)
#endif
   ?? " seconds"
   SET CONSOLE OFF
   if print
      SET PRINTER ON
      ?? " "
      FOR ii = pcol +1 TO 61
         ?? "."
      NEXT
      ?? str(loctime,7,2), "seconds"
      SET PRINTER OFF
   endif
ENDIF
RETURN NIL

*** eof

