/* ********************** dbedit.prg ****************************************
(c) 1998 by multisoft Datentechnik GmbH, Munich, Germany. All rights reserved.

Note: you may modify this file and distribute it in object form (by including
      it in the executable) with your application. You may not distribute the
      changed or unchanged file for other commercial purposes.

Compile: FlagShip -c -m -na -w dbedit.prg [-odbedit.o]
Link   : FlagShip myappl*.prg dbedit.o ...

Additions to DBEDIT() functionality of FlagShip 4.3 and Clipper 5.x:
- Fields may also be specified as "field1;field2;...;fieldN" string
- Optional parameters cForCond|bForCond, cWhiCond|bWhiCond, nNext
- Three (instead of two) parameters are passed to the UDF. The 3rd
  parameter is the TBrowse object itself.
- On start-up, the UDF is invoked with mode 5 (DE_START) to allow
  modify the Tbrowse object.

************************ start of DBEDIT() ******************************** */

#ifdef FlagShip
# define FlagShip44                 /* use prototyping. Disable for rel 4.3 */
# include "inkey.fh"
# include "setcurs.fh"
# include "dbedit.fh"
# include "dbstruct.fh"
#else
# include "inkey.ch"
# include "setcurs.ch"
# include "dbedit.ch"
# include "dbstruct.ch"
# define  LOCAL_INT LOCAL
#endif
#ifdef FlagShip44
# include "tbrclass.fh"                        /* TBrowse  class prototypes */
#endif

** #define ALERNATIVE_ERROR

#define TRUE        .T.
#define FALSE       .F.

#ifndef DE_APPEND
#define DE_APPEND    3
#endif

#ifndef DE_START
#define DE_START     5                     /* new in FS4.4            */
#endif

#ifndef RTARGVAL                           /* == FSerrors.h           */
#define RTARGVAL     313                   /* Wrong argument value    */
#define RTNODBF      322                   /* No database opened      */
#endif

#define RTEMSG1 "Wrong type/value of parameter "
#define RTEMSG2 "No open database in area "
#define RTEMSG3 "Field '"
#define RTEMSG4 "' not available"
#define RTEMSG5 "1 or 3"
#define RTEMSG6 "2 or 6"
#define RTEMSG7 "13 or 14"

*------------------------------------------------------------------------
*
FUNCTION DBEdit (nTop, nLeft, nBott, nRight, axFields, cbxUserUdf, ;
                 axPict, axHeadTxt, axHeadSep, axColSep, axFootSep, ;
                 axFootTxt, cbForCond, cbWhiCond, nNext)

LOCAL aFields, aHeadTxt, aPict, aHeadSep, aColSep, aFootSep, aFootTxt
LOCAL cbUserUdf, saveCurs, lHasUDF := FALSE
LOCAL saveRow := ROW(), saveCol := COL(), aStruct
LOCAL nn, bNIL := {|| NIL}
LOCAL nRow, nCol, udfRet, nKey, bKeyBlck
LOCAL lAppend := .F.
LOCAL iSkip := 0, xSkip := 0, nStartRec := recno()
LOCAL xFirst := 0, xLast := 0, xMax := lastrec()
LOCAL CurRec := 0
LOCAL_INT ii, iArrLen := 0

#ifdef FlagShip
#ifdef FlagShip44
 LOCAL oTbr  AS TBrowse             // prototyped for FlagShip 4.4
 LOCAL oTcol AS TBColumn            // for compile-time resolution
#else
 LOCAL oTbr, oTcol                  // for FlagShip 4.3x
#endif
 LOCAL dummy, dummy2                // general for FlagShip
#else
 LOCAL oTbr, oTcol                  // otherwise, for Clipper
 MEMVAR  dummy, dummy2
 PRIVATE dummy, dummy2
#endif

* ------------------------------------
* Parameter checking, set defaults
* ------------------------------------

nTop  := max (default (nTop,  "N", 0), 0)
nLeft := max (default (nLeft, "N", 0), 0)
nBott := min (default (nBott, "N", maxrow()), maxrow())
nRight:= min (default (nRight,"N", maxcol()), maxcol())

if nTop >= nBott
   RTError (RTARGVAL, RTEMSG1 + RTEMSG5)
   return NIL
endif
if nLeft >= nRight
   RTError (RTARGVAL, RTEMSG1 + RTEMSG6)
   return NIL
endif
if !used()
   RTError (RTNODBF, RTEMSG2 + ltrim(str(select())))
   return NIL
endif
cbUserUdf := cbxUserUdf

if !(valtype(cbForCond) $ "CMBU") .or. !(valtype(cbWhiCond) $ "CMBU")
   RTError (RTARGVAL, RTEMSG1 + RTEMSG7)
   return NIL
endif
if !(valtype(nNext) $ "NIFU")
   RTError (RTARGVAL, RTEMSG1 + "15")
   return NIL
endif

if valtype(cbForCond) $ "CM"
   cbForCond := &("{||" + cbForCond + "}")    // convert to code block
endif
if valtype(cbWhiCond) $ "CM"
   cbWhiCond := &("{||" + cbWhiCond + "}")    // convert to code block
endif

* ------------------------------------
* Fields
* ------------------------------------
aFields := {}
IF VALTYPE(axFields) == "C" .and. !empty(axFields)
   WHILE (ii := at(";", axFields)) > 0
      dummy := alltrim(upper(substr(axFields, 1, ii-1)))
      if !empty(dummy)
         aadd (aFields, dummy)
      endif
      axFields := substr(axFields, ii+1)
   ENDDO
   if !empty(axFields)
      aadd (aFields, alltrim(upper(axFields)))
   endif
elseif VALTYPE(axFields) == "A"
   FOR ii := 1 to LEN(axFields)
      IF VALTYPE(axFields[ii]) == "C" .and. !empty(axFields[ii])
         aadd (afields, upper(alltrim(axFields[ii])))
      ENDIF
   NEXT
else
   aStruct := DBSTRUCT()
   IF LEN(aStruct)  == 0       // default to all fields in current work area
      RETURN NIL               // error, nothing to work with
   ENDIF
   FOR ii := 1 TO LEN(aStruct)
      AADD(aFields,aStruct[ii,DBS_NAME])
   NEXT
endif
iArrLen := LEN(aFields)
if iArrLen == 0
   RTError (RTARGVAL, RTEMSG1 + "5")
   return NIL
endif
* -------------------------------------
* column headings, footings, separators
* -------------------------------------
IF VALTYPE(axHeadTxt) == "C"
   aHeadTxt := AFILL(ARRAY(iArrLen),axHeadTxt)
else                                             // create aHeadTxt[]
   aHeadTxt := ACLONE (if (VALTYPE(axHeadTxt) == "A", axHeadTxt, aFields))
   ASIZE(aHeadTxt,iArrLen)
   for ii := 1 to iArrLen
      if VALTYPE(aHeadTxt[ii]) != "C"
         aHeadTxt[ii] := aFields[ii]
      endif
   next
ENDIF
if VALTYPE(axFootTxt) == "C" .and. len(axFootTxt) > 0
   aFootTxt := AFILL(ARRAY(iArrLen),axFootTxt)
elseif valtype(axFootTxt) == "A"                 // create aFootTxt[]
   aFootTxt := ACLONE (axFootTxt)
   ASIZE(aFootTxt,iArrLen)
   for ii := 1 to iArrLen
      if VALTYPE(aFootTxt[ii]) != "C"
         aFootTxt[ii] := ""
      endif
   next
endif
if VALTYPE(axHeadSep) == "C"
   aHeadSep := axHeadSep
elseif valtype(axHeadSep) == "A"                 // create aHeadSep[]
   aHeadSep := ACLONE (axHeadSep)
   ASIZE(aHeadSep,iArrLen)
   for ii := 1 to iArrLen
      if VALTYPE(aHeadSep[ii]) != "C"
         aHeadSep[ii] := NIL
      endif
   next
else
   aHeadSep := NIL
endif

if VALTYPE(axColSep) == "C"
   aColSep := axColSep
elseif valtype(axColSep) == "A"                 // create aColSep[]
   aColSep := ACLONE (axColSep)
   ASIZE(aColSep,iArrLen)
   for ii := 1 to iArrLen
      if VALTYPE(aColSep[ii]) != "C"
         aColSep[ii] := NIL
      endif
   next
else
   aColSep := NIL
endif
if VALTYPE(axFootSep) == "C"
   aFootSep := axFootSep
elseif valtype(axFootSep) == "A"                 // create aFootSep[]
   aFootSep := ACLONE (axFootSep)
   ASIZE(aFootSep,iArrLen)
   for ii := 1 to iArrLen
      if VALTYPE(aFootSep[ii]) != "C"
         aFootSep[ii] := NIL
      endif
   next
else
   aFootSep := NIL
endif
if VALTYPE(axPict) == "C" .and. !empty(axPict)   // create aPict[]
   aPict := AFILL(ARRAY(iArrLen),axPict)
elseif valtype(axPict) == "A"
   aPict := ACLONE (axPict)
   ASIZE(aPict,iArrLen)
else
   aPict := array(iArrLen)
endif
for ii := 1 to iArrLen
   if VALTYPE(aPict[ii]) != "C" .or. empty(aPict[ii])
      aPict[ii] := NIL
   endif
next

* --------------------------------------------------------------------------
* Create Tbrowse object
* --------------------------------------------------------------------------
oTbr := TBrowseDB(nTop, nLeft, nBott, nRight)    // create Tbrowse object

if cbForCond != NIL .or. cbWhiCond != NIL .or. nNext != NIL
   oTbr:GoTopBlock    := {|| MyGoTop (cbForCond, cbWhiCond, nNext, ;
                             nStartRec, @xFirst, @iSkip) }
   oTbr:GoBottomBlock := {|| MyGoBottom(lAppend, cbForCond, cbWhiCond, nNext,;
                             nStartRec, @xLast, xMax, @xSkip, @iSkip)}
endif

oTbr:SKIPBLOCK := {|nn| ;
    SkipDbedit(nn, lAppend, cbForCond, cbWhiCond, nNext, @iSkip)}


* --------------------------------------------------------------------------
* Add a column to the object for each field specified in the array of fields
* --------------------------------------------------------------------------
FOR ii := 1 to iArrLen
   oTcol := TBColumnNew(aHeadTxt[ii], bNIL)      // create TBcolumn object
   dummy := aFields[ii]
   if TYPE(dummy) == "M"
      oTcol:BLOCK := {|| IF(EMPTY(&dummy),"<memo>","<MEMO>")}
   elseif TYPE(dummy) == "U"
      RTError (RTARGVAL, RTEMSG3 + dummy + RTEMSG4)
      return NIL
   elseif valtype(aPict[ii]) == "C" .and. !empty(aPict[ii])
      dummy2:= "'" + aPict[ii] + "'"                   // corrected
      oTcol:BLOCK := {|| TRANSFORM(&dummy, &dummy2) }  // corrected
   else
      oTcol:BLOCK := {|| &dummy }
   endif

   if valtype(aHeadSep) == "A"
      if valtype(aHeadSep[ii]) == "C"
         oTcol:HEADSEP := aHeadSep[ii]
      endif
   endif
   if valtype(aColSep) == "A"
      if valtype(aColSep[ii]) == "C"
         oTcol:COLSEP := aColSep[ii]
      endif
   endif
   if valtype(aFootSep) == "A"
      if valtype(aFootSep[ii]) == "C"
         oTcol:FOOTSEP := aFootSep[ii]
      endif
   endif
   if valtype(aFootTxt) == "A"
      if valtype(aFootTxt[ii]) == "C"
         oTcol:FOOTING := aFootTxt[ii]
      endif
   endif
   oTbr:addColumn(oTcol)
NEXT

if aHeadSep == NIL
   oTbr:HEADSEP := ""
elseif valtype(aHeadSep) == "C"
   oTbr:HEADSEP := aHeadSep
endif
if aColSep == NIL
   oTbr:COLSEP := "  "
elseif VALTYPE(aColSep) == "C"
   oTbr:COLSEP := aColSep
endif
if VALTYPE(aFootSep) == "C"
   oTbr:FOOTSEP := aFootSep
endif
if VALTYPE(aFootTxt) == "C"
   oTbr:FOOTING := aFootTxt
endif

if VALTYPE(cbUserUdf) == "B"
   lHasUDF := TRUE
elseif VALTYPE(cbUserUdf) == "C"
   dummy := cbUserUdf
   cbUserUdf := {|x,y,z| &dummy (x,y,z)}  // turn user UDF into a code block
   lHasUDF := TRUE
endif

saveCurs := SETCURSOR(0)                 // set cursor off

* -----------------------------------------------------
* Everthing has been initialized, run the browse object
* -----------------------------------------------------
dbcommit()                               // flush dbf buffers
IF lHasUDF
   IF EVAL(cbUserUdf, DE_START, oTbr:colPos, oTbr) == DE_ABORT
      RETURN NIL
   ENDIF
ENDIF

* find first matching FOR condition, if WHILE is not given

if cbForCond != NIL .and. cbWhiCond == NIL
   while !eof() .and. !eval(cbForCond)
     if valtype(nNext) != "U"
        if iSkip >= (nNext -1)
           exit
        endif
     endif
     skip
     iSkip++
   enddo
endif
nStartRec := recno()

* main control loop

do while TRUE
   if ( oTbr:COLPOS <= oTbr:FREEZE )     // Don't allow cursor to
      oTbr:COLPOS := oTbr:FREEZE + 1     // move into frozen columns
   endif
   while (NEXTKEY() == 0) .AND. (! oTbr:STABILIZE())       // stabilize
   enddo

   if oTbr:STABLE .and. lHasUDF
      nRow := ROW()
      nCol := COL()
      DEVPOS(nRow,nCol)

      nn := DE_IDLE
      if lastrec() == 0
         nn := DE_EMPTY
      elseif oTbr:HITTOP
         nn := DE_HITTOP
      elseif oTbr:HITBOTTOM
         nn := DE_HITBOTTOM
      endif
      udfRet := EVAL(cbUserUdf, nn, oTbr:COLPOS, oTbr)   // call UDF

      * The UDF may now GO to EOF() (GOTO BOTTOM, SKIP) and return DE_APPEND
      * to signal DBEDIT() appending new record

      DO CASE
      CASE udfRet == DE_ABORT                    // 0: perform quit
         EXIT
      CASE udfRet == DE_CONT                     // 1: continue
         oTbr:REFRESHCURRENT()
      CASE udfRet == DE_REFRESH                  // 2: repaint the screen
         lAppend := .F.
         DoRefreshAll (oTbr)
      CASE udfRet == DE_APPEND                   // 3: append new
         if recno() <= lastrec() .or. !lAppend
            if !lAppend
               lAppend := .T.
            else
               oTbr:REFRESHCURRENT()
               oTbr:FORCESTABLE()
               go bottom
            endif
            oTbr:DOWN()
         endif
         oTbr:FORCESTABLE()

         * The .dbf is now on EOF, the UDF is entered with the DE_IDLE
         * mode. The UDF may now APPEND BLANK if required. If so,
         * the append mode is finished. Otherwise, the editing can be
         * done on exception key, by entering the UDF with DE_EXCEPT,
         * the UDF should perform "keyboard chr(K_ENTER)"

         udfRet := EVAL(cbUserUdf, DE_IDLE, oTbr:colPos, oTbr)
         if !eof() .and. udfRet # DE_APPEND
            lAppend := .F.
            DoRefreshAll (oTbr)
         endif
      OTHERWISE
         oTbr:REFRESHCURRENT()
         ?? chr(7)
      ENDCASE

      WHILE (NEXTKEY() == 0) .AND. (! oTbr:STABILIZE())    // stabilize
      ENDDO
   endif
*  ---------------------------
*  wait for key and process it
*  ---------------------------
   nKey := INKEY(0)

   IF ((bKeyBlck := SETKEY(nKey)) != NIL )       // SET KEY function ?
      EVAL(bKeyBlck, PROCNAME(2), PROCLINE(2), READVAR())
   ENDIF

   DO CASE
   CASE  nKey == K_UP
      if lAppend
         lAppend := .F.
         doRefreshAll (oTbr)
      else
         oTbr:UP()
      endif
   CASE  nKey == K_DOWN
      if lAppend
         oTbr:HITBOTTOM := .T.
      else
         oTbr:down()
      endif
   CASE  nKey == K_LEFT
      oTbr:left()
   CASE  nKey == K_RIGHT
      oTbr:right()
   CASE  nKey == K_PGUP
      if lAppend
         lAppend := .F.
         doRefreshAll (oTbr)
      else
         oTbr:PAGEUP()
      endif
   CASE  nKey == K_PGDN
      if lAppend
         oTbr:HITBOTTOM := .T.
      else
         oTbr:pageDown()
      endif
   CASE  nKey == K_HOME
      oTbr:home()
   CASE  nKey == K_END
      oTbr:end()
   CASE  nKey == K_CTRL_LEFT .OR. nKey == K_SH_TAB
      oTbr:panLeft()
   CASE  nKey == K_CTRL_RIGHT .OR. nKey == K_TAB
      oTbr:panRight()
   CASE  nKey == K_CTRL_HOME
      oTbr:panHome()
   CASE  nKey == K_CTRL_END
      oTbr:panEnd()
   CASE  nKey == K_CTRL_PGUP
      if lAppend
         lAppend := .F.
         doRefreshAll (oTbr)
      else
         oTbr:GOTOP()
      endif
   CASE  nKey == K_CTRL_PGDN
      if lAppend
         lAppend := .F.
         doRefreshAll (oTbr)
      else
         oTbr:GOBOTTOM()
      endif
   OTHERWISE
      IF lHasUDF
         oTbr:FORCESTABLE()

         * The UDF is now entered with the DE_EXCEPT mode. It has itself to
         * perform the edititing and APPEND BLANK + REPLACE -> DE_REFRESH
         * otherwise return DE_CONTINUE to remain in append mode

         udfRet := EVAL(cbUserUdf,DE_EXCEPT,oTbr:colPos,oTbr)
         DO CASE
         CASE udfRet == DE_ABORT                    // 0: perform quit
            EXIT
         CASE udfRet == DE_REFRESH                  // 2: repaint the screen
            if lAppend .or. eof()
               lAppend := .F.
               go bottom
            endif
            doRefreshAll (oTbr)
         OTHERWISE                                  // 1 or 3
            oTbr:REFRESHCURRENT()
         ENDCASE
      ELSE
         IF nKey == K_ENTER .OR. nKey == K_ESC
            EXIT
         ENDIF
      ENDIF
   ENDCASE
ENDDO

SETCURSOR(saveCurs)
SETPOS   (saveRow, saveCol)                      // restore cursor
RETURN NIL

******************************************
STATIC FUNCTION SkipDbedit(nMode, lAppend, ;
                cbForCond, cbWhiCond, nNext, iSkip)  // called from skip block
******************************************
LOCAL nCount := 0, lFound := .T., nLast := recno(), xSkip := iSkip
Local isFor := valtype(cbForCond) == "B"
Local isWhi := valtype(cbWhiCond) == "B"
Local isNext:= valtype(nNext) != "U"

if LASTREC() == 0
   RETURN 0
endif

DO CASE
CASE nMode == 0
   skip 0
CASE nMode < 0
   WHILE nCount > nMode
      if isNext .and. iSkip <= 0                 // consider NEXT
         exit
      endif
      SKIP -1
      if isWhi .and. !eval(cbWhiCond)            // consider WHILE
         exit
      endif
      if isFor                                   // consider FOR
         lFound := eval(cbForCond)
      endif
      if lFound
         nLast := recno()
      endif
      if BOF()
         exit
      endif
      iSkip--
      if lFound
         nCount--
         xskip := iskip
      endif
   ENDDO
   goto nLast
   iSkip := xSkip

CASE recno() <=  lastrec()
   while nCount < nMode
      if isNext .and. iSkip >= (nNext -1)        // consider NEXT
         exit
      endif
      SKIP 1
      if eof()
         if lAppend
            nCount++
            nLast := recno()
         else
            goto nLast
         endif
         exit
      endif
      if isWhi .and. !eval(cbWhiCond)            // consider WHILE
         goto nLast
         exit
      endif
      if isFor                                   // consider FOR
         lFound := eval(cbForCond)
      endif
      nLast := if (lFound, recno(), nLast)
      iSkip++
      if lFound
         nCount++
         xskip := iSkip
      endif
   enddo
   goto nLast
   iSkip := xSkip
ENDCASE
return (nCount)

******************************************
STATIC FUNCTION MyGoBottom(lAppend, cbForCond, cbWhiCond, nNext, ;
                           nStart, xLast, xMax, xSkip, iSkip)
LOCAL lFound := .T., nLast := recno()
Local isFor := valtype(cbForCond) == "B", ;
      isWhi := valtype(cbWhiCond) == "B", ;
      isNext:= valtype(nNext) != "U"
Local xxSkip := iSkip

if lastrec() = 0
   return .F.
elseif xlast > 0 .and. xMax == lastrec() .and. ;
   (isdbexcl() .or. indexord() == 0) .and. !lAppend
   goto xLast                                    // optimize:
   iSkip := xSkip                                // already found
elseif isNext
   while !eof() .and. iSkip < (nNext -1)         // NEXT given
      skip
      if isWhi .and. !eval(cbWhiCond)            // consider WHILE
         goto nLast
         exit
      endif
      if isFor                                   // consider FOR
         lFound := eval(cbForCond)
      endif
      iskip++
      if lFound
         nLast := recno()
         xxSkip := iSkip
      endif
   enddo
   goto nLast
   iSkip := xxSkip
   xSkip := iskip
elseif isWhi                                     // WHILE given
   while !eof() .and. eval(cbWhiCond)
      skip
      if isFor                                   // consider FOR
         lFound := eval(cbForCond)
      endif
      iskip++
      if lFound
         nLast := recno()
      endif
   enddo
   goto nLast
elseif isFor                                     // FOR given
   goto bottom
   while !bof() .and. !eval(cbForCond)
      skip -1
   enddo
else
   goto bottom
   xLast := recno()
   if lAppend
      skip
   endif
endif
if !lAppend
   xLast := recno()
endif
return .T.

******************************************
STATIC FUNCTION MyGoTop(cbForCond, cbWhiCond, ;
                        nNext, nStart, xFirst, iSkip)
LOCAL lFound := .T., nLast := nStart
Local isFor := valtype(cbForCond) == "B", ;
      isWhi := valtype(cbWhiCond) == "B", ;
      isNext:= valtype(nNext) != "U"

if lastrec() = 0
   return .F.
elseif xFirst > 0 .and. (isdbexcl() .or. indexord() == 0)  // optimize:
   goto xFirst                                             // already found
   iskip := 0
elseif isNext
   goto nStart
   iSkip := 0
elseif isWhi
   goto nStart
   while !bof() .and. eval(cbWhiCond)
      skip -1
      if isFor                                   // consider FOR
         lFound := eval(cbForCond)
      endif
      nLast := if (lFound, recno(), nLast)
   enddo
   goto nLast
elseif isFor
   goto top
   while !eof()
      if eval(cbForCond)
         exit
      endif
      skip
   enddo
else
   goto top
endif
xFirst := recno()

return .T.

***********************************
#ifdef FlagShip44
static FUNCTION doRefreshAll (oTbr AS TBrowse)   // FS 4.4, prototypes
#else
static FUNCTION doRefreshAll (oTbr)              // refresh all & index
#endif
***********************************
LOCAL oldrec := recno()

oTbr:REFRESHALL()
oTbr:FORCESTABLE()

if oldrec > lastrec()
   return NIL
endif

while recno() != oldrec .and. !oTbr:HITTOP
   oTbr:UP()
   oTbr:FORCESTABLE()
enddo

return NIL

*******************************************
static FUNCTION default (var, type, defval)      // set default values
*******************************************
if valtype(type) == "C"
   return if (valtype(var) == upper(type), var, defval)
endif
return if (var == NIL, defval, var)

*************************************
static FUNCTION RTError (errVal, str)            // call error handler
*************************************
LOCAL errBlock, oErr

#ifndef ES_RTERR
#define ES_RTERR 2
#endif

#ifdef ALERNATIVE_ERROR
   if alert ("DBEDIT() " + str + ";in " + ;
             procname(2) + "/" + ltrim(str(procline(2))), ;
             {"Quit","Continue"}) == 1
      quit
   endif
#else
   errBlock := ErrorBlock()                  // default error handler block
   oErr     := ErrorNew()                    // new error object

   oErr:ARGS          := NIL
   oErr:CANDEFAULT    := .F.
   oErr:CANRETRY      := .F.
   oErr:CANSUBSTITUTE := .F.
   oErr:CARGO         := NIL
   oErr:DESCRIPTION   := "DBEDIT() " + str
   oErr:FILENAME      := PROCNAME(2) + "(" + LTRIM(STR(PROCLINE(2),5)) + ")"
   oErr:GENCODE       := errVal
   oErr:OPERATION     := ""
   oErr:OSCODE        := DOSERROR()
   oErr:SEVERITY      := ES_RTERR
   oErr:SUBCODE       := 0
   oErr:SUBSYSTEM     := "BASE"

   EVAL (errBlock, oErr)
#endif
return NIL

*************************************
#ifndef FlagShip
static function isdbexcl()
return .T.
#endif

*** EOF *** DBEDIT.PRG
